@@ -51,21 +51,22 @@
 #endif
 #include <errno.h>
 #include <stdarg.h>
-#include <string>
-#include <map>
+
 #include <algorithm>  // for sort(), equal(), and copy()
+#include <map>
+#include <memory>
+#include <string>
 
 #include "heap-profile-table.h"
 
-#include "base/logging.h"
-#include "raw_printer.h"
-#include "symbolize.h"
-#include <gperftools/stacktrace.h>
-#include <gperftools/malloc_hook.h>
-#include "memory_region_map.h"
 #include "base/commandlineflags.h"
-#include "base/logging.h"    // for the RawFD I/O commands
+#include "base/logging.h"
+#include "base/proc_maps_iterator.h"
 #include "base/sysinfo.h"
+#include "gperftools/malloc_hook.h"
+#include "gperftools/stacktrace.h"
+#include "memory_region_map.h"
+#include "symbolize.h"
 
 using std::sort;
 using std::equal;
@@ -73,9 +74,6 @@ using std::copy;
 using std::string;
 using std::map;
 
-using tcmalloc::FillProcSelfMaps;   // from sysinfo.h
-using tcmalloc::DumpProcSelfMaps;   // from sysinfo.h
-
 //----------------------------------------------------------------------
 
 DEFINE_bool(cleanup_old_heap_profiles,
@@ -111,13 +109,6 @@ static const int kStripFrames = 2;
 static const int kStripFrames = 3;
 #endif
 
-// For sorting Stats or Buckets by in-use space
-static bool ByAllocatedSpace(HeapProfileTable::Stats* a,
-                             HeapProfileTable::Stats* b) {
-  // Return true iff "a" has more allocated space than "b"
-  return (a->alloc_size - a->free_size) > (b->alloc_size - b->free_size);
-}
-
 //----------------------------------------------------------------------
 
 HeapProfileTable::HeapProfileTable(Allocator alloc,
@@ -274,164 +265,47 @@ void HeapProfileTable::MarkAsIgnored(const void* ptr) {
   }
 }
 
-// We'd be happier using snprintfer, but we don't to reduce dependencies.
-int HeapProfileTable::UnparseBucket(const Bucket& b,
-                                    char* buf, int buflen, int bufsize,
-                                    const char* extra,
-                                    Stats* profile_stats) {
-  if (profile_stats != NULL) {
-    profile_stats->allocs += b.allocs;
-    profile_stats->alloc_size += b.alloc_size;
-    profile_stats->frees += b.frees;
-    profile_stats->free_size += b.free_size;
-  }
-  int printed =
-    snprintf(buf + buflen, bufsize - buflen, "%6" PRId64 ": %8" PRId64 " [%6" PRId64 ": %8" PRId64 "] @%s",
-             b.allocs - b.frees,
-             b.alloc_size - b.free_size,
-             b.allocs,
-             b.alloc_size,
-             extra);
-  // If it looks like the snprintf failed, ignore the fact we printed anything
-  if (printed < 0 || printed >= bufsize - buflen) return buflen;
-  buflen += printed;
+void HeapProfileTable::UnparseBucket(const Bucket& b,
+                                     tcmalloc::GenericWriter* writer,
+                                     const char* extra) {
+  writer->AppendF("%6" PRId64 ": %8" PRId64 " [%6" PRId64 ": %8" PRId64 "] @",
+                  b.allocs - b.frees,
+                  b.alloc_size - b.free_size,
+                  b.allocs,
+                  b.alloc_size);
+  writer->AppendStr(extra);
+
   for (int d = 0; d < b.depth; d++) {
-    printed = snprintf(buf + buflen, bufsize - buflen, " 0x%08" PRIxPTR,
-                       reinterpret_cast<uintptr_t>(b.stack[d]));
-    if (printed < 0 || printed >= bufsize - buflen) return buflen;
-    buflen += printed;
+    writer->AppendF(" 0x%08" PRIxPTR,
+                    reinterpret_cast<uintptr_t>(b.stack[d]));
   }
-  printed = snprintf(buf + buflen, bufsize - buflen, "\n");
-  if (printed < 0 || printed >= bufsize - buflen) return buflen;
-  buflen += printed;
-  return buflen;
+  writer->AppendStr("\n");
 }
 
-HeapProfileTable::Bucket**
-HeapProfileTable::MakeSortedBucketList() const {
-  Bucket** list = static_cast<Bucket**>(alloc_(sizeof(Bucket) * num_buckets_));
-
-  int bucket_count = 0;
-  for (int i = 0; i < kHashTableSize; i++) {
-    for (Bucket* curr = bucket_table_[i]; curr != 0; curr = curr->next) {
-      list[bucket_count++] = curr;
-    }
-  }
-  RAW_DCHECK(bucket_count == num_buckets_, "");
-
-  sort(list, list + num_buckets_, ByAllocatedSpace);
-
-  return list;
-}
-
-void HeapProfileTable::IterateOrderedAllocContexts(
-    AllocContextIterator callback) const {
-  Bucket** list = MakeSortedBucketList();
-  AllocContextInfo info;
-  for (int i = 0; i < num_buckets_; ++i) {
-    *static_cast<Stats*>(&info) = *static_cast<Stats*>(list[i]);
-    info.stack_depth = list[i]->depth;
-    info.call_stack = list[i]->stack;
-    callback(info);
-  }
-  dealloc_(list);
-}
-
-int HeapProfileTable::FillOrderedProfile(char buf[], int size) const {
-  Bucket** list = MakeSortedBucketList();
-
-  // Our file format is "bucket, bucket, ..., bucket, proc_self_maps_info".
-  // In the cases buf is too small, we'd rather leave out the last
-  // buckets than leave out the /proc/self/maps info.  To ensure that,
-  // we actually print the /proc/self/maps info first, then move it to
-  // the end of the buffer, then write the bucket info into whatever
-  // is remaining, and then move the maps info one last time to close
-  // any gaps.  Whew!
-  int map_length = snprintf(buf, size, "%s", kProcSelfMapsHeader);
-  if (map_length < 0 || map_length >= size) {
-      dealloc_(list);
-      return 0;
-  }
-  bool dummy;   // "wrote_all" -- did /proc/self/maps fit in its entirety?
-  map_length += FillProcSelfMaps(buf + map_length, size - map_length, &dummy);
-  RAW_DCHECK(map_length <= size, "");
-  char* const map_start = buf + size - map_length;      // move to end
-  memmove(map_start, buf, map_length);
-  size -= map_length;
-
-  Stats stats;
-  memset(&stats, 0, sizeof(stats));
-  int bucket_length = snprintf(buf, size, "%s", kProfileHeader);
-  if (bucket_length < 0 || bucket_length >= size) {
-      dealloc_(list);
-      return 0;
-  }
-  bucket_length = UnparseBucket(total_, buf, bucket_length, size,
-                                " heapprofile", &stats);
+void HeapProfileTable::SaveProfile(tcmalloc::GenericWriter* writer) const {
+  writer->AppendStr(kProfileHeader);
+  UnparseBucket(total_, writer, " heapprofile");
 
   // Dump the mmap list first.
   if (profile_mmap_) {
-    BufferArgs buffer(buf, bucket_length, size);
     MemoryRegionMap::LockHolder holder{};
-    MemoryRegionMap::IterateBuckets<BufferArgs*>(DumpBucketIterator, &buffer);
-    bucket_length = buffer.buflen;
+    MemoryRegionMap::IterateBuckets([writer] (const Bucket* bucket) {
+      UnparseBucket(*bucket, writer, "");
+    });
   }
 
-  for (int i = 0; i < num_buckets_; i++) {
-    bucket_length = UnparseBucket(*list[i], buf, bucket_length, size, "",
-                                  &stats);
-  }
-  RAW_DCHECK(bucket_length < size, "");
-
-  dealloc_(list);
-
-  RAW_DCHECK(buf + bucket_length <= map_start, "");
-  memmove(buf + bucket_length, map_start, map_length);  // close the gap
-
-  return bucket_length + map_length;
-}
-
-// static
-void HeapProfileTable::DumpBucketIterator(const Bucket* bucket,
-                                          BufferArgs* args) {
-  args->buflen = UnparseBucket(*bucket, args->buf, args->buflen, args->bufsize,
-                               "", NULL);
-}
-
-inline
-void HeapProfileTable::DumpNonLiveIterator(const void* ptr, AllocValue* v,
-                                           const DumpArgs& args) {
-  if (v->live()) {
-    v->set_live(false);
-    return;
-  }
-  if (v->ignore()) {
-    return;
-  }
-  Bucket b;
-  memset(&b, 0, sizeof(b));
-  b.allocs = 1;
-  b.alloc_size = v->bytes;
-  b.depth = v->bucket()->depth;
-  b.stack = v->bucket()->stack;
-  char buf[1024];
-  int len = UnparseBucket(b, buf, 0, sizeof(buf), "", args.profile_stats);
-  RawWrite(args.fd, buf, len);
-}
-
-// Callback from NonLiveSnapshot; adds entry to arg->dest
-// if not the entry is not live and is not present in arg->base.
-void HeapProfileTable::AddIfNonLive(const void* ptr, AllocValue* v,
-                                    AddNonLiveArgs* arg) {
-  if (v->live()) {
-    v->set_live(false);
-  } else {
-    if (arg->base != NULL && arg->base->map_.Find(ptr) != NULL) {
-      // Present in arg->base, so do not save
-    } else {
-      arg->dest->Add(ptr, *v);
+  int bucket_count = 0;
+  for (int i = 0; i < kHashTableSize; i++) {
+    for (Bucket* curr = bucket_table_[i]; curr != nullptr; curr = curr->next) {
+      UnparseBucket(*curr, writer, "");
+      bucket_count++;
     }
   }
+  RAW_DCHECK(bucket_count == num_buckets_, "");
+  (void)bucket_count;
+
+  writer->AppendStr(kProcSelfMapsHeader);
+  tcmalloc::SaveProcSelfMaps(writer);
 }
 
 bool HeapProfileTable::WriteProfile(const char* file_name,
@@ -443,14 +317,33 @@ bool HeapProfileTable::WriteProfile(const char* file_name,
     RAW_LOG(ERROR, "Failed dumping filtered heap profile to %s", file_name);
     return false;
   }
-  RawWrite(fd, kProfileHeader, strlen(kProfileHeader));
-  char buf[512];
-  int len = UnparseBucket(total, buf, 0, sizeof(buf), " heapprofile", NULL);
-  RawWrite(fd, buf, len);
-  const DumpArgs args(fd, NULL);
-  allocations->Iterate<const DumpArgs&>(DumpNonLiveIterator, args);
+
+  tcmalloc::RawFDGenericWriter<> writer{fd};
+
+  writer.AppendStr(kProfileHeader);
+
+  UnparseBucket(total, &writer, " heapprofile");
+
+  allocations->Iterate([&writer] (const void* ptr, AllocValue* v) {
+    if (v->live()) {
+      v->set_live(false);
+      return;
+    }
+    if (v->ignore()) {
+      return;
+    }
+    Bucket b;
+    memset(&b, 0, sizeof(b));
+    b.allocs = 1;
+    b.alloc_size = v->bytes;
+    b.depth = v->bucket()->depth;
+    b.stack = v->bucket()->stack;
+    UnparseBucket(b, &writer, "");
+  });
+
   RawWrite(fd, kProcSelfMapsHeader, strlen(kProcSelfMapsHeader));
-  DumpProcSelfMaps(fd);
+  tcmalloc::SaveProcSelfMapsToRawFD(fd);
+
   RawClose(fd);
   return true;
 }
@@ -481,7 +374,9 @@ void HeapProfileTable::CleanupOldProfiles(const char* prefix) {
 
 HeapProfileTable::Snapshot* HeapProfileTable::TakeSnapshot() {
   Snapshot* s = new (alloc_(sizeof(Snapshot))) Snapshot(alloc_, dealloc_);
-  address_map_->Iterate(AddToSnapshot, s);
+  address_map_->Iterate([s] (const void* ptr, AllocValue* v) {
+    s->Add(ptr, *v);
+  });
   return s;
 }
 
@@ -490,12 +385,6 @@ void HeapProfileTable::ReleaseSnapshot(Snapshot* s) {
   dealloc_(s);
 }
 
-// Callback from TakeSnapshot; adds a single entry to snapshot
-void HeapProfileTable::AddToSnapshot(const void* ptr, AllocValue* v,
-                                     Snapshot* snapshot) {
-  snapshot->Add(ptr, *v);
-}
-
 HeapProfileTable::Snapshot* HeapProfileTable::NonLiveSnapshot(
     Snapshot* base) {
   RAW_VLOG(2, "NonLiveSnapshot input: %" PRId64 " %" PRId64 "\n",
@@ -503,10 +392,17 @@ HeapProfileTable::Snapshot* HeapProfileTable::NonLiveSnapshot(
            total_.alloc_size - total_.free_size);
 
   Snapshot* s = new (alloc_(sizeof(Snapshot))) Snapshot(alloc_, dealloc_);
-  AddNonLiveArgs args;
-  args.dest = s;
-  args.base = base;
-  address_map_->Iterate<AddNonLiveArgs*>(AddIfNonLive, &args);
+  address_map_->Iterate([&] (const void* ptr, AllocValue* v) {
+    if (v->live()) {
+      v->set_live(false);
+    } else {
+      if (base != nullptr && base->map_.Find(ptr) != nullptr) {
+        // Present in arg->base, so do not save
+      } else {
+        s->Add(ptr, *v);
+      }
+    }
+  });
   RAW_VLOG(2, "NonLiveSnapshot output: %" PRId64 " %" PRId64 "\n",
            s->total_.allocs - s->total_.frees,
            s->total_.alloc_size - s->total_.free_size);
@@ -526,22 +422,6 @@ struct HeapProfileTable::Snapshot::Entry {
   }
 };
 
-// State used to generate leak report.  We keep a mapping from Bucket pointer
-// the collected stats for that bucket.
-struct HeapProfileTable::Snapshot::ReportState {
-  map<Bucket*, Entry> buckets_;
-};
-
-// Callback from ReportLeaks; updates ReportState.
-void HeapProfileTable::Snapshot::ReportCallback(const void* ptr,
-                                                AllocValue* v,
-                                                ReportState* state) {
-  Entry* e = &state->buckets_[v->bucket()]; // Creates empty Entry first time
-  e->bucket = v->bucket();
-  e->count++;
-  e->bytes += v->bytes;
-}
-
 void HeapProfileTable::Snapshot::ReportLeaks(const char* checker_name,
                                              const char* filename,
                                              bool should_symbolize) {
@@ -555,19 +435,24 @@ void HeapProfileTable::Snapshot::ReportLeaks(const char* checker_name,
           size_t(total_.allocs));
 
   // Group objects by Bucket
-  ReportState state;
-  map_.Iterate(&ReportCallback, &state);
+  std::map<Bucket*, Entry> buckets;
+  map_.Iterate([&] (const void* ptr, AllocValue* v) {
+    Entry* e = &buckets[v->bucket()]; // Creates empty Entry first time
+    e->bucket = v->bucket();
+    e->count++;
+    e->bytes += v->bytes;
+  });
 
   // Sort buckets by decreasing leaked size
-  const int n = state.buckets_.size();
+  const int n = buckets.size();
   Entry* entries = new Entry[n];
   int dst = 0;
-  for (map<Bucket*,Entry>::const_iterator iter = state.buckets_.begin();
-       iter != state.buckets_.end();
+  for (map<Bucket*,Entry>::const_iterator iter = buckets.begin();
+       iter != buckets.end();
        ++iter) {
     entries[dst++] = iter->second;
   }
-  sort(entries, entries + n);
+  std::sort(entries, entries + n);
 
   // Report a bounded number of leaks to keep the leak report from
   // growing too long.
@@ -584,21 +469,26 @@ void HeapProfileTable::Snapshot::ReportLeaks(const char* checker_name,
       symbolization_table.Add(e.bucket->stack[j]);
     }
   }
-  static const int kBufSize = 2<<10;
-  char buffer[kBufSize];
   if (should_symbolize)
     symbolization_table.Symbolize();
-  for (int i = 0; i < to_report; i++) {
-    const Entry& e = entries[i];
-    base::RawPrinter printer(buffer, kBufSize);
-    printer.Printf("Leak of %zu bytes in %d objects allocated from:\n",
-                   e.bytes, e.count);
-    for (int j = 0; j < e.bucket->depth; j++) {
-      const void* pc = e.bucket->stack[j];
-      printer.Printf("\t@ %" PRIxPTR " %s\n",
-          reinterpret_cast<uintptr_t>(pc), symbolization_table.GetSymbol(pc));
+
+  {
+    auto do_log = +[] (const char* buf, size_t amt) {
+      RAW_LOG(ERROR, "%.*s", amt, buf);
+    };
+    constexpr int kBufSize = 2<<10;
+    tcmalloc::WriteFnWriter<decltype(do_log), kBufSize> printer{do_log};
+
+    for (int i = 0; i < to_report; i++) {
+      const Entry& e = entries[i];
+      printer.AppendF("Leak of %zu bytes in %d objects allocated from:\n",
+                      e.bytes, e.count);
+      for (int j = 0; j < e.bucket->depth; j++) {
+        const void* pc = e.bucket->stack[j];
+        printer.AppendF("\t@ %" PRIxPTR " %s\n",
+                        reinterpret_cast<uintptr_t>(pc), symbolization_table.GetSymbol(pc));
+      }
     }
-    RAW_LOG(ERROR, "%s", buffer);
   }
 
   if (to_report < n) {
@@ -607,22 +497,15 @@ void HeapProfileTable::Snapshot::ReportLeaks(const char* checker_name,
   }
   delete[] entries;
 
-  // TODO: Dump the sorted Entry list instead of dumping raw data?
-  // (should be much shorter)
   if (!HeapProfileTable::WriteProfile(filename, total_, &map_)) {
     RAW_LOG(ERROR, "Could not write pprof profile to %s", filename);
   }
 }
 
-void HeapProfileTable::Snapshot::ReportObject(const void* ptr,
-                                              AllocValue* v,
-                                              char* unused) {
-  // Perhaps also log the allocation stack trace (unsymbolized)
-  // on this line in case somebody finds it useful.
-  RAW_LOG(ERROR, "leaked %zu byte object %p", v->bytes, ptr);
-}
-
 void HeapProfileTable::Snapshot::ReportIndividualObjects() {
-  char unused;
-  map_.Iterate(ReportObject, &unused);
+  map_.Iterate([] (const void* ptr, AllocValue* v) {
+    // Perhaps also log the allocation stack trace (unsymbolized)
+    // on this line in case somebody finds it useful.
+    RAW_LOG(ERROR, "leaked %zu byte object %p", v->bytes, ptr);
+  });
 }
