@@ -38,7 +38,7 @@
 
 #include <stddef.h>
 #include <stdint.h>
-#ifdef HAVE_SYS_SYSCALL_H
+#if HAVE_SYS_SYSCALL_H
 #include <sys/syscall.h>
 #endif
 
@@ -49,17 +49,9 @@
 #include <algorithm>
 #include "base/logging.h"
 #include "base/spinlock.h"
+#include "malloc_backtrace.h"
 #include "maybe_emergency_malloc.h"
 
-// This #ifdef should almost never be set.  Set NO_TCMALLOC_SAMPLES if
-// you're porting to a system where you really can't get a stacktrace.
-#ifdef NO_TCMALLOC_SAMPLES
-  // We use #define so code compiles even if you #include stacktrace.h somehow.
-# define GetStackTrace(stack, depth, skip)  (0)
-#else
-# include <gperftools/stacktrace.h>
-#endif
-
 // __THROW is defined in glibc systems.  It means, counter-intuitively,
 // "This function will never throw an exception."  It's an optional
 // optimization tool, but we may need to use it to match glibc prototypes.
@@ -129,7 +121,7 @@ void InitialNewHook(const void* ptr, size_t size) {
 // may assume that the initial malloc hooks have been removed.  The initializers
 // may set up malloc hooks and allocate memory.
 bool RemoveInitialHooksAndCallInitializers() {
-  static tcmalloc::TrivialOnce once{base::LINKER_INITIALIZED};
+  static tcmalloc::TrivialOnce once;
   once.RunOnce([] () {
     RAW_CHECK(MallocHook::RemoveNewHook(&InitialNewHook), "");
   });
@@ -148,7 +140,7 @@ namespace base { namespace internal {
 // The potential for contention is very small.  This needs to be a SpinLock and
 // not a Mutex since it's possible for Mutex locking to allocate memory (e.g.,
 // per-thread allocation in debug builds), which could cause infinite recursion.
-static SpinLock hooklist_spinlock(base::LINKER_INITIALIZED);
+static SpinLock hooklist_spinlock;
 
 template <typename T>
 bool HookList<T>::Add(T value) {
@@ -323,14 +315,11 @@ void MallocHook::InvokeDeleteHookSlow(const void* p) {
 
 #undef INVOKE_HOOKS
 
-#ifndef NO_TCMALLOC_SAMPLES
+#if !defined(NO_TCMALLOC_SAMPLES) && HAVE_ATTRIBUTE_SECTION_START
 
 DEFINE_ATTRIBUTE_SECTION_VARS(google_malloc);
 DECLARE_ATTRIBUTE_SECTION_VARS(google_malloc);
   // actual functions are in debugallocation.cc or tcmalloc.cc
-DEFINE_ATTRIBUTE_SECTION_VARS(malloc_hook);
-DECLARE_ATTRIBUTE_SECTION_VARS(malloc_hook);
-  // actual functions are in this file, malloc_hook.cc, and low_level_alloc.cc
 
 #define ADDR_IN_ATTRIBUTE_SECTION(addr, name) \
   (reinterpret_cast<uintptr_t>(ATTRIBUTE_SECTION_START(name)) <= \
@@ -342,8 +331,7 @@ DECLARE_ATTRIBUTE_SECTION_VARS(malloc_hook);
 // that calls one of our hooks via MallocHook:Invoke*.
 // A helper for GetCallerStackTrace.
 static inline bool InHookCaller(const void* caller) {
-  return ADDR_IN_ATTRIBUTE_SECTION(caller, google_malloc) ||
-         ADDR_IN_ATTRIBUTE_SECTION(caller, malloc_hook);
+  return ADDR_IN_ATTRIBUTE_SECTION(caller, google_malloc);
   // We can use one section for everything except tcmalloc_or_debug
   // due to its special linkage mode, which prevents merging of the sections.
 }
@@ -360,12 +348,6 @@ static inline void CheckInHookCaller() {
       RAW_LOG(ERROR, "google_malloc section is missing, "
                      "thus InHookCaller is broken!");
     }
-    INIT_ATTRIBUTE_SECTION_VARS(malloc_hook);
-    if (ATTRIBUTE_SECTION_START(malloc_hook) ==
-        ATTRIBUTE_SECTION_STOP(malloc_hook)) {
-      RAW_LOG(ERROR, "malloc_hook section is missing, "
-                     "thus InHookCaller is broken!");
-    }
     checked_sections = true;
   }
 }
@@ -384,7 +366,7 @@ extern "C" int MallocHook_GetCallerStackTrace(void** result, int max_depth,
   // Note: this path is inaccurate when a hook is not called directly by an
   // allocation function but is daisy-chained through another hook,
   // search for MallocHook::(Get|Set|Invoke)* to find such cases.
-  return GetStackTrace(result, max_depth, skip_count + int(DEBUG_MODE));
+  return tcmalloc::GrabBacktrace(result, max_depth, skip_count + int(DEBUG_MODE));
   // due to -foptimize-sibling-calls in opt mode
   // there's no need for extra frame skip here then
 #else
@@ -398,7 +380,7 @@ extern "C" int MallocHook_GetCallerStackTrace(void** result, int max_depth,
     // and 3 is to account for some hook daisy chaining.
   static const int kStackSize = kMaxSkip + 1;
   void* stack[kStackSize];
-  int depth = GetStackTrace(stack, kStackSize, 1);  // skip this function frame
+  int depth = tcmalloc::GrabBacktrace(stack, kStackSize, 1);  // skip this function frame
   if (depth == 0)   // silenty propagate cases when GetStackTrace does not work
     return 0;
   for (int i = 0; i < depth; ++i) {  // stack[0] is our immediate caller
@@ -420,7 +402,7 @@ extern "C" int MallocHook_GetCallerStackTrace(void** result, int max_depth,
       if (depth < max_depth  &&  depth + i == kStackSize) {
         // get frames for the missing depth
         depth +=
-          GetStackTrace(result + depth, max_depth - depth, 1 + kStackSize);
+          tcmalloc::GrabBacktrace(result + depth, max_depth - depth, 1 + kStackSize);
       }
       return depth;
     }
