@@ -31,13 +31,13 @@
 
 // A test for low_level_alloc.cc
 
+#include "base/low_level_alloc.h"
+
 #include <stdio.h>
 #include <map>
-#include "base/low_level_alloc.h"
-#include "base/logging.h"
-#include <gperftools/malloc_hook.h>
 
-using std::map;
+#include <gperftools/malloc_hook.h>
+#include "gtest/gtest.h"
 
 // a block of memory obtained from the allocator
 struct BlockDesc {
@@ -50,7 +50,7 @@ struct BlockDesc {
 // by RandomizeBlockDesc is still there.
 static void CheckBlockDesc(const BlockDesc &d) {
   for (int i = 0; i != d.len; i++) {
-    CHECK((d.ptr[i] & 0xff) == ((d.fill + i) & 0xff));
+    ASSERT_TRUE((d.ptr[i] & 0xff) == ((d.fill + i) & 0xff));
   }
 }
 
@@ -63,10 +63,6 @@ static void RandomizeBlockDesc(BlockDesc *d) {
   }
 }
 
-// Use to indicate to the malloc hooks that
-// this calls is from LowLevelAlloc.
-static bool using_low_level_alloc = false;
-
 // n times, toss a coin, and based on the outcome
 // either allocate a new block or deallocate an old block.
 // New blocks are placed in a map with a random key
@@ -76,19 +72,15 @@ static bool using_low_level_alloc = false;
 // before being freed.  At the end of the run,
 // all remaining allocated blocks are freed.
 // If use_new_arena is true, use a fresh arena, and then delete it.
-// If call_malloc_hook is true and user_arena is true,
-// allocations and deallocations are reported via the MallocHook
-// interface.
-static void Test(bool use_new_arena, bool call_malloc_hook, int n) {
-  typedef map<int, BlockDesc> AllocMap;
+static void ExerciseAllocator(bool use_new_arena, int n) {
+  typedef std::map<int, BlockDesc> AllocMap;
   AllocMap allocated;
   AllocMap::iterator it;
   BlockDesc block_desc;
   int rnd;
   LowLevelAlloc::Arena *arena = 0;
   if (use_new_arena) {
-    int32 flags = call_malloc_hook?  LowLevelAlloc::kCallMallocHook :  0;
-    arena = LowLevelAlloc::NewArena(flags, LowLevelAlloc::DefaultArena());
+    arena = LowLevelAlloc::NewArena(nullptr);
   }
   for (int i = 0; i != n; i++) {
     if (i != 0 && i % 10000 == 0) {
@@ -98,22 +90,18 @@ static void Test(bool use_new_arena, bool call_malloc_hook, int n) {
 
     switch(rand() & 1) {      // toss a coin
     case 0:     // coin came up heads: add a block
-      using_low_level_alloc = true;
       block_desc.len = rand() & 0x3fff;
       block_desc.ptr =
         reinterpret_cast<char *>(
                         arena == 0
                         ? LowLevelAlloc::Alloc(block_desc.len)
                         : LowLevelAlloc::AllocWithArena(block_desc.len, arena));
-      using_low_level_alloc = false;
       RandomizeBlockDesc(&block_desc);
       rnd = rand();
       it = allocated.find(rnd);
       if (it != allocated.end()) {
         CheckBlockDesc(it->second);
-        using_low_level_alloc = true;
         LowLevelAlloc::Free(it->second.ptr);
-        using_low_level_alloc = false;
         it->second = block_desc;
       } else {
         allocated[rnd] = block_desc;
@@ -123,9 +111,7 @@ static void Test(bool use_new_arena, bool call_malloc_hook, int n) {
       it = allocated.begin();
       if (it != allocated.end()) {
         CheckBlockDesc(it->second);
-        using_low_level_alloc = true;
         LowLevelAlloc::Free(it->second.ptr);
-        using_low_level_alloc = false;
         allocated.erase(it);
       }
       break;
@@ -134,62 +120,17 @@ static void Test(bool use_new_arena, bool call_malloc_hook, int n) {
   // remove all remaniing blocks
   while ((it = allocated.begin()) != allocated.end()) {
     CheckBlockDesc(it->second);
-    using_low_level_alloc = true;
     LowLevelAlloc::Free(it->second.ptr);
-    using_low_level_alloc = false;
     allocated.erase(it);
   }
   if (use_new_arena) {
-    CHECK(LowLevelAlloc::DeleteArena(arena));
+    ASSERT_TRUE(LowLevelAlloc::DeleteArena(arena));
   }
 }
 
-// used for counting allocates and frees
-static int32 allocates;
-static int32 frees;
-
-// called on each alloc if kCallMallocHook specified
-static void AllocHook(const void *p, size_t size) {
-  if (using_low_level_alloc) {
-    allocates++;
+TEST(LowLevelAllocTest, Basic) {
+  ExerciseAllocator(false, 50000);
+  for (int i = 0; i < 8; i++) {
+    ExerciseAllocator(true, 15000);
   }
 }
-
-// called on each free if kCallMallocHook specified
-static void FreeHook(const void *p) {
-  if (using_low_level_alloc) {
-    frees++;
-  }
-}
-
-int main(int argc, char *argv[]) {
-  if (argc != 1) {
-    fprintf(stderr, "USAGE: %s\n", argv[0]);
-    return 1;
-  }
-
-  CHECK(MallocHook::AddNewHook(&AllocHook));
-  CHECK(MallocHook::AddDeleteHook(&FreeHook));
-  CHECK_EQ(allocates, 0);
-  CHECK_EQ(frees, 0);
-  Test(false, false, 50000);
-  CHECK_NE(allocates, 0);   // default arena calls hooks
-  CHECK_NE(frees, 0);
-  for (int i = 0; i != 16; i++) {
-    bool call_hooks = ((i & 1) == 1);
-    allocates = 0;
-    frees = 0;
-    Test(true, call_hooks, 15000);
-    if (call_hooks) {
-      CHECK_GT(allocates, 5000); // arena calls hooks
-      CHECK_GT(frees, 5000);
-    } else {
-      CHECK_EQ(allocates, 0);    // arena doesn't call hooks
-      CHECK_EQ(frees, 0);
-    }
-  }
-  printf("\nPASS\n");
-  CHECK(MallocHook::RemoveNewHook(&AllocHook));
-  CHECK(MallocHook::RemoveDeleteHook(&FreeHook));
-  return 0;
-}
